package oysteikt.calc;

import java.util.List;
import java.util.function.BinaryOperator;
import java.util.function.UnaryOperator;

import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.Labeled;
import javafx.scene.control.ListView;

public class CalcController {

  private Calc calc;

  public CalcController() {
    calc = new Calc(0.0, 0.0, 0.0);
  }

  public Calc getCalc() {
    return calc;
  }

  public void setCalc(Calc calc) {
    this.calc = calc;
    updateOperandsView();
  }

  @FXML
  private ListView<Double> operandsView;

  @FXML
  private Label operandView;

  @FXML
  void initialize() {
    updateOperandsView();
  }

  private void updateOperandsView() {
    List<Double> operands = operandsView.getItems();
    operands.clear();
    int elementCount = Math.min(calc.getOperandCount(), 3);
    for (int i = 0; i < elementCount; i++) {
      operands.add(calc.peekOperand(elementCount - i - 1));
    }
  }

  private String getOperandString() {
    return operandView.getText();
  }

  private boolean hasOperand() {
    return !getOperandString().isBlank();
  }

  private double getOperand() {
    return Double.valueOf(operandView.getText());
  }

  private void setOperand(String operandString) {
    operandView.setText(operandString);
  }

  @FXML
  void handleEnter() {
    if (hasOperand()) {
      calc.pushOperand(getOperand());
    } else {
      calc.dup();
    }
    setOperand("");
    updateOperandsView();
  }

  /* -------------------------------------------------------------------------- */
  /*                             Exercise Functions                             */
  /* -------------------------------------------------------------------------- */

  protected void appendToOperand(String s) {
    this.setOperand(this.getOperandString() + s);
  }

  @FXML
  void handleDigit(ActionEvent ae) {
    if (ae.getSource() instanceof Labeled l) {
      this.appendToOperand(l.getText());
    }
  }

  @FXML
  void handlePoint() {
    var operandString = this.getOperandString();
    if (operandString.contains(".")) {
      String stringBeforeDot = operandString.substring(0, operandString.indexOf('.') + 1);
      this.setOperand(stringBeforeDot);
    } else {
      this.appendToOperand(".");
    }
  }

  @FXML
  void handleClear() {
    this.setOperand("");
  }

  private void pushOperandIfPossible() {
    if (this.hasOperand()) {
      this.calc.pushOperand(this.getOperand());
      this.setOperand("");
    }
  }

  @FXML
  void handleSwap() {
    this.pushOperandIfPossible();
    this.calc.swap();
    this.updateOperandsView();
  }

  private void performOperation(UnaryOperator<Double> op) {
    this.pushOperandIfPossible();
    this.calc.performOperation(op);
    this.updateOperandsView();
  }

  // Removed `swap` parameter as described in 
  // https://piazza.com/class/kssraggjcxm4at?cid=56
  private void performOperation(BinaryOperator<Double> op) {
    if (this.calc.getOperandCount() > 3) { // There should always be at least 3 times 0.0 in the calculator
      this.pushOperandIfPossible();
      this.calc.performOperation(op);
      this.updateOperandsView();
    }
  }

  @FXML
  void handleOpAdd() {
    this.performOperation((x, y) -> x + y);
  }

  @FXML
  void handleOpSub() {
    this.performOperation((x, y) -> x - y);
  }

  @FXML
  void handleOpMult() {
    this.performOperation((x, y) -> x * y);
  }

  @FXML
  void handleOpDiv() {
    this.performOperation((x, y) -> x / y);
  }

  @FXML
  void handleOpRoot() {
    this.performOperation((x) -> Math.sqrt(x));
  }

  @FXML
  void handleOpPi() {
    this.calc.pushOperand(Math.PI);
    this.updateOperandsView();
  }
}
